@@ -69,6 +69,15 @@
  * Server side.
  */
 
+struct msgb_alloc_info {
+		/*! Whether it was set by user or we use iofd defaults */
+		bool set_by_user;
+		/*! size of msgb to allocate (excluding headroom) */
+		unsigned int size;
+		/*! headroom to allocate when allocating msgb's */
+		unsigned int headroom;
+};
+
 #define OSMO_STREAM_SRV_F_RECONF	(1 << 0)
 #define OSMO_STREAM_SRV_F_NODELAY	(1 << 1)
 
@@ -88,6 +97,7 @@ struct osmo_stream_srv_link {
 	void			*data;
 	int			flags;
 	unsigned int		tx_queue_max_length; /* Max amount of msgbs which can be enqueued */
+	struct msgb_alloc_info	msgb_alloc;
 	struct osmo_sock_init2_multiaddr_pars ma_pars;
 };
 
@@ -356,10 +366,10 @@ int osmo_stream_srv_link_set_type(struct osmo_stream_srv_link *link, int type)
 	return 0;
 }
 
-/*! Set the socket type for the stream server link.
+/*! Set the socket domain for the stream server link.
  *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
- *  \param[in] type Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
+ *  \param[in] domain Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
  *  \returns zero on success, negative on error.
  */
 int osmo_stream_srv_link_set_domain(struct osmo_stream_srv_link *link, int domain)
@@ -506,6 +516,26 @@ void osmo_stream_srv_link_set_accept_cb(struct osmo_stream_srv_link *link,
 	link->accept_cb = accept_cb;
 }
 
+/*! Set the msgb allocation parameters on child osmo_stream_srv objects
+ *  \param[in] link Stream Server Link
+ *  \param[in] size Size of msgb to allocate (excluding headroom)
+ *  \param[in] headroom Headroom to allocate when allocating msgb's
+ *
+ *  The parameters are applied to osmo_stream_srv objects upon creation.
+ *  Setting both to 0 leaves it as implementation default.
+ **/
+int osmo_stream_srv_link_set_msgb_alloc_info(struct osmo_stream_srv_link *link, unsigned int size, unsigned int headroom)
+{
+	if (size == 0 && headroom == 0) {
+		link->msgb_alloc.set_by_user = false;
+	} else {
+		link->msgb_alloc.set_by_user = true;
+		link->msgb_alloc.headroom = headroom;
+		link->msgb_alloc.size = size;
+	}
+	return 0;
+}
+
 /*! Destroy the stream server link. Closes + Releases Memory.
  *  \param[in] link Stream Server Link */
 void osmo_stream_srv_link_destroy(struct osmo_stream_srv_link *link)
@@ -962,6 +992,8 @@ osmo_stream_srv_create2(void *ctx, struct osmo_stream_srv_link *link, int fd, vo
 	}
 
 	osmo_iofd_set_txqueue_max_length(conn->iofd, conn->srv->tx_queue_max_length);
+	if (conn->srv->msgb_alloc.set_by_user)
+		osmo_iofd_set_alloc_info(conn->iofd, conn->srv->msgb_alloc.size, conn->srv->msgb_alloc.headroom);
 
 	if (osmo_iofd_register(conn->iofd, fd) < 0) {
 		LOGSSRV(conn, LOGL_ERROR, "could not register FD %d\n", fd);
@@ -1072,15 +1104,15 @@ static void configure_srv_segmentation_cb(struct osmo_stream_srv *conn)
  *  \param[in,out] conn Target Stream Server to modify
  *  \param[in] segmentation_cb Segmentation callback to be set
  *
- *  DEPRECATED: Use osmo_stream_srv_set_segmentation_cb2() instead.
+ *  A segmentation call-back can optionally be used when a packet based protocol
+ *  (like TCP) is used within a STREAM style socket that does not preserve
+ *  message boundaries within the stream.  If a segmentation call-back is given,
+ *  the osmo_stream_srv library code will makes sure that the read_cb called
+ *  only for complete single messages, and not arbitrary segments of the stream.
  *
- *  A segmentation call-back can optionally be used when a packet based protocol (like TCP) is used within a
- *  STREAM style socket that does not preserve message boundaries within the stream.  If a segmentation
- *  call-back is given, the osmo_stream_srv library code will makes sure that the read_cb called only for
- *  complete single messages, and not arbitrary segments of the stream.
- *
- *  This function only works with osmo_stream_srv in osmo_io mode, created by osmo_stream_srv_create2()!
- *  The connection has to have been established prior to calling this function.
+ *  This function only works with osmo_stream_srv in osmo_io mode, created by
+ *  osmo_stream_srv_create2()! The connection has to have been established prior
+ *  to calling this function.
  *
  */
 void osmo_stream_srv_set_segmentation_cb(struct osmo_stream_srv *conn,
@@ -1098,16 +1130,12 @@ void osmo_stream_srv_set_segmentation_cb(struct osmo_stream_srv *conn,
  *  \param[in,out] conn Target Stream Server to modify
  *  \param[in] segmentation_cb2 Segmentation callback to be set
  *
- *  A segmentation call-back can optionally be used when a packet based protocol (like TCP) is used within a
- *  STREAM style socket that does not preserve message boundaries within the stream.  If a segmentation
- *  call-back is given, the osmo_stream_srv library code will makes sure that the read_cb called only for
- *  complete single messages, and not arbitrary segments of the stream.
- *
- *  This function only works with osmo_stream_srv in osmo_io mode, created by osmo_stream_srv_create2()!
- *  The connection has to have been established prior to calling this function.
+ * Same as osmo_stream_srv_set_segmentation_cb(), but a
+ * osmo_stream_srv_segmentation_cb2_t is called instead which allows access to
+ * the related conn object.
  */
 void osmo_stream_srv_set_segmentation_cb2(struct osmo_stream_srv *conn,
-					 osmo_stream_srv_segmentation_cb2_t segmentation_cb2)
+					  osmo_stream_srv_segmentation_cb2_t segmentation_cb2)
 {
 	/* Note that the following implies that iofd != NULL, since
 	 * osmo_stream_srv_create2() creates the iofd member, too */
